/*
   Copyright 2009,2010 Ricardo Cañuelo Navarro

   This file is part of Oolong.

   Oolong is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Oolong is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Oolong.  If not, see <http://www.gnu.org/licenses/>.
*/

package es.rediris.forja.tea.connector;

import javax.xml.namespace.QName;
import javax.xml.ws.Service;

import java.io.*;
import java.net.*;
import java.util.*;

import es.rediris.forja.tea.api.*;
import es.rediris.forja.tea.server.Sensor;

/**
 * This class retrieves services from a Tea station and then it
 * serves the retrieved data to the Oolong client
 * @author Ricardo Cañuelo Navarro
 */
public class OolongConnector
{
	/**
	 * Entry point. Creates a new OolongConnector and
	 * calls its go method
	 */
	public static void main(String[] args)
	{
		new OolongConnector().go();
	}
	
	/**
	 * Starts the connector
	 */
	public void go()
	{
		try {
			ServerSocket serverSock = new ServerSocket(9001);
			while(true) {
				System.out.println("Esperando conexion");
				Socket clientSocket = serverSock.accept();
				System.out.println("Socket creado");
				Thread t = new Thread(new ClientHandler(clientSocket));
				t.start();
				System.out.println("Hebra iniciada");
			}
		} catch(Exception ex) {
			ex.printStackTrace();
		}
	}
	
	/**
	 * A Runnable which handles the client request
	 * @author rcn
	 */
	public class ClientHandler implements Runnable
	{
		Socket sock;
		ObjectInputStream input;
		ObjectOutputStream output;
		Request req;
		
		public ClientHandler(Socket clientSocket)
		{
			try {
				sock = clientSocket;
				input = new ObjectInputStream(sock.getInputStream());
				System.out.println("Conectado a Oolong");
				output = new ObjectOutputStream(sock.getOutputStream());
			} catch (Exception ex) {
				ex.printStackTrace();
			}			
		}
		
		/**
		 * Reads the request object through the socket,
		 * Makes up a service request,
		 * Sends the service data through the socket
		 */
		public void run()
		{
			req = new Request();
			boolean disconnect = false;
			while(!disconnect) {
				try {
					req = (Request)input.readObject();
					System.out.println("Recibido objeto Request");
					URL url = new URL(req.url);
					QName qname = new QName(req.serverName, req.serviceName);
					Service service = Service.create(url, qname);		
					Sensor sensor = service.getPort(Sensor.class);
					System.out.println("Obtenido servicio web");
					System.out.println(sensor.getAttribute());
					Response answer = new Response();
					answer.value = sensor.getValue();
					output.writeObject(answer);
				} catch(Exception ex) {
					disconnect = true;
				}
			}
		}
	}
}
