/*
   Copyright 2009,2010 Ricardo Cañuelo Navarro

   This file is part of Oolong.

   Oolong is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Oolong is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Oolong.  If not, see <http://www.gnu.org/licenses/>.
*/

package es.rediris.forja.tea.plugins.temperature;

import es.rediris.forja.tea.api.*;
import javax.swing.*;
import java.net.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;

/**
 * A class for creating a Thermometer object
 * @author rcn
 *
 */
public class Thermometer implements Plugin
{
	/* Instance variables */
	String magnitude = "Temperature";
	float value = 0;
	String server;
	String port;
	boolean stop = false;

	JFrame frame;
	JThermometer display;
	ImageIcon icon;

	Thread reader;
	Request req;
	Socket sock;
	ObjectOutputStream output;
	ObjectInputStream input;
	
	public Thermometer()
	{
		frame = new JFrame(magnitude);
		display = new JThermometer();
		icon = new ImageIcon(getClass().getResource("termometro_logo.png"));
		frame.setContentPane(display);
		frame.addWindowListener(new closeListener());
	}
	
	public Plugin copy()
	{
		Thermometer thermCopy = null;
		thermCopy = new Thermometer();
		return thermCopy;
	}
	
	public void go()
	{
		setUpNetworking();
		frame.pack();
		frame.setResizable(false);
		frame.setVisible(true);
	}

	public void setServer(String s)
	{
		server = s;
	}

	public void setPort(String p)
	{
		port = p;
	}	

	public void setUpNetworking()
	{
		try {
			sock = new Socket("127.0.0.1", 9001);
			output = new ObjectOutputStream(sock.getOutputStream());
			input = new ObjectInputStream(sock.getInputStream());
			System.out.println("Conectado al conector Oolong");
			req = new Request();
			req.url = server + ":" + port + "/ts?wsdl";
			req.serverName = "http://server.tea.forja.rediris.es/";
			req.serviceName = "TempSensorService";
		} catch(Exception ex) {
			ex.printStackTrace();
		}
		reader = new Thread(new ServiceReader());
		reader.start();
	}
	
	/* Meter interface methods */
	
	public String getMagnitude()
	{
		return magnitude;
	}

	public float getValue()
	{
		return value;
	}

	public ImageIcon getIcon()
	{
		return icon;
	}

	public void setMagnitude(String mag)
	{
		magnitude=mag;
	}

	public void setValue(float val)
	{
		value=val;
		this.getDisplay().setValue(val);
	}

	public JThermometer getDisplay()
	{
		return display;
	}
	
	/**
	 * Returns this object's String value 
	 * @return The String value of this object
	 */
	public String toString()
	{
		return "Thermometer";
	}

	class ServiceReader implements Runnable
	{
		public void run()
		{
			while(!stop) {
				try {
					output.writeObject(req);
					Response res = (Response)input.readObject();
					setValue(res.value);
					Thread.sleep(3000);
				} catch(Exception ex) {
					ex.printStackTrace();
				}
			}
			try {
				output.close();
			} catch(Exception ex) {
				ex.printStackTrace();
			}
		}
	}

	class timerListener implements ActionListener
	{
		public void actionPerformed (ActionEvent evt)
		{
			reader = new Thread(new ServiceReader());
			reader.start();
		}
	}

	class closeListener implements WindowListener
	{
		public void windowClosing(WindowEvent evt)
		{
			stop = true;
		}

		public void windowDeactivated(WindowEvent evt) {}
		public void windowActivated(WindowEvent evt) {}
		public void windowDeiconified(WindowEvent evt) {}
		public void windowIconified(WindowEvent evt) {}
		public void windowClosed(WindowEvent evt) {}
		public void windowOpened(WindowEvent evt) {}
	}
}
